//
//  main.m
//
//  This file is a part of librabbitmq-objc-examples
//
//  Copyright © 2012 Jani Hur <http://www.jani-hur.net>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#import <Foundation/Foundation.h>

#import "AMQPConnection.h"
#import "AMQPChannel.h"
#import "AMQPQueue.h"
#import "AMQPConsumer.h"
#import "AMQPMessage.h"

#import "GetOpt.h"

int main(int argc, const char * argv[])
{
    //
    // A signal handling with GCD (Grand Central Dispatch)
    // http://mikeash.com/pyblog//friday-qa-2011-04-01-signal-handling.html
    //
    
    // A flag modified when a signal is received
    __block Boolean acceptMessages = true;
    
    // GCD dispatch source
    // SIGINT = ^C
    dispatch_source_t source = dispatch_source_create(DISPATCH_SOURCE_TYPE_SIGNAL, SIGINT, 0, dispatch_get_global_queue(0, 0));
    
    // Set a code block (^{}) to be executed when a signal arrives
    dispatch_source_set_event_handler(source, ^{
        NSLog(@"Received signal SIGINT, going to quit after next message");
        acceptMessages = false;
    });
    dispatch_resume(source);
    
    // sigaction has to be told to ignore the signal processed with GCD
    {
        struct sigaction action = { 0 };
        action.sa_handler = SIG_IGN;
        sigaction(SIGINT, &action, NULL);
    }
    
    @autoreleasepool {
        
        @try {
            NSLog(@"Starting an AMQP consumer");

            GetOpt* opts = [[GetOpt alloc]init];
            NSLog(@"opts = %@", opts);
            
            NSString* host = [opts getStringOf:@"-h"];
            NSString* routingkey = [opts getStringOf:@"-r"];

            AMQPConnection* connection = [[AMQPConnection alloc] init];
            
            [connection connectToHost:host];
            [connection login];
            
            AMQPChannel* channel = [connection openChannel];
            
            AMQPQueue* queue = [[AMQPQueue alloc] initWithName:routingkey onChannel:channel isPassive:FALSE isExclusive:FALSE isDurable:FALSE getsAutoDeleted:TRUE];
            
            AMQPConsumer* consumer = [[AMQPConsumer alloc] initForQueue:queue onChannel:channel useAcknowledgements:TRUE isExclusive:TRUE receiveLocalMessages:NO];
            
            while (acceptMessages) {
                AMQPMessage* message = [consumer pop];
                
                NSLog(@"message = %@", message);
                NSLog(@"message.body = %@", message.body);
            }
            
            NSLog(@"Closing an AMQP consumer");
        }
        @catch (NSException* ex) {
            NSLog(@"EXCEPTION %@: %@", [ex name], [ex reason]);
        }
        
    }
    return 0;
}


