//
//  main.m
//
//  This file is a part of librabbitmq-objc-examples
//
//  Copyright © 2012 Jani Hur <http://www.jani-hur.net>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#import <Foundation/Foundation.h>

#import "AMQPConnection.h"
#import "AMQPExchange.h"
#import "AMQPQueue.h"
#import "AMQPConsumer.h"
#import "AMQPMessage.h"

#import "GetOpt.h"

int main(int argc, const char * argv[])
{
    @autoreleasepool {
        @try {
            //
            // command line options
            //
            
            GetOpt* opts = [[GetOpt alloc]init];
            NSLog(@"opts = %@", opts);
            
            NSString* host = [opts getStringOf:@"-h"];
            NSString* routingkey = [opts getStringOf:@"-r"];
            NSString* message = [opts getStringOf:@"-m"];

            //
            // establish a connection
            //
            
            AMQPConnection* connection = [[AMQPConnection alloc] init];
            
            [connection connectToHost:host];
            [connection login];
            
            //
            // build reply receiver
            //
            
            AMQPChannel* replyChannel = [connection openChannel];
            AMQPQueue* replyQueue = [[AMQPQueue alloc]initReplyQueue:replyChannel];
            AMQPConsumer* consumer = [[AMQPConsumer alloc] initForQueue:replyQueue onChannel:replyChannel useAcknowledgements:TRUE isExclusive:TRUE receiveLocalMessages:NO];
            
            //
            // build an initial message and send it
            //
            
            AMQPChannel* channel = [connection openChannel];
            
            amqp_basic_properties_t props;
            props._flags = AMQP_BASIC_CONTENT_TYPE_FLAG | AMQP_BASIC_DELIVERY_MODE_FLAG | AMQP_BASIC_REPLY_TO_FLAG | AMQP_BASIC_CORRELATION_ID_FLAG;
            props.content_type = amqp_cstring_bytes("text/plain");
            props.delivery_mode = AMQP_PERSISTENT_DELIVERY_MODE;
            props.reply_to = amqp_cstring_bytes([[replyQueue getQueueName]UTF8String]);
            NSString* correlationId = [[NSProcessInfo processInfo] globallyUniqueString];
            props.correlation_id = amqp_cstring_bytes([correlationId UTF8String]);
            
            AMQPExchange* exchange = [[AMQPExchange alloc] initWithName:@"" onChannel:channel];

            [exchange publishMessage:message usingRoutingKey:routingkey usingProperties:&props];

            //
            // wait for a reply
            //
            
            AMQPMessage* replyMessage = [consumer pop];
            
            NSLog(@"replyMessage = %@", replyMessage);
            NSLog(@"replyMessage.body = %@", replyMessage.body);
            
            //
            // check if correlation id matches
            //
            
            if ([replyMessage.correlationID isEqualToString: correlationId]) {
                NSLog(@"Correlation id in reply message matched");
            } else {
                NSLog(@"Correlation id in reply message didn't matched");
            }
        }
        @catch (NSException *exception) {
            NSLog(@"EXCEPTION %@: %@", [exception name], [exception reason]);
        }
    }
    return 0;
}
